import pytest
import os
import json
import uuid
import time
from tenacity import retry, stop_after_attempt, wait_fixed, retry_if_exception_type

from psc_access_integration.api_clients.adminservices import AdminServicesApiAccess
from psc_auth_integration.entities.auth_user_session import AuthUserSession
from assignments.common.utils import DEFENSE_URL, USERNAME, PASSWORD, ENV, sim_endpoint, \
    create_asset_group, \
    add_policy_override, add_asset_group_to_endpoint, LOCAL_PAS_HOST, EXPECTED_CALL_TO_SUCCEED, execute, \
    delete_sensor, delete_policy, delete_asset_group, remove_asset_group_from_endpoint
from policy_service_integration.api_client.policyservice import PolicyApi
from python_sensor_sim.enums import SensorDeviceType

from assignments.tests.test_policy_preview import search_assets

SESSION = None

CREATED_ASSET_GROUP = None
CREATED_POLICY = None
CREATED_ENDPOINT = None


# @pytest.mark.assignment
def test_reevaluate():
    try:
        global CREATED_ASSET_GROUP, CREATED_POLICY, CREATED_ENDPOINT, SESSION
        os.environ["ENV"] = ENV
        SESSION = AuthUserSession(gateway_url=DEFENSE_URL, username=USERNAME, password=PASSWORD)

        policy_api = PolicyApi(SESSION)
        response = policy_api.get_policies_summary(org_key=SESSION.get_user_org_key())
        assert response.ok, EXPECTED_CALL_TO_SUCCEED
        policies = json.loads(response.content).get("policies")

        response = policy_api.get_policy_details(org_key=SESSION.get_user_org_key(), policy_id=policies[0].get("id"))
        assert response.ok, EXPECTED_CALL_TO_SUCCEED
        policy_details = json.loads(response.content)

        policy_details['name'] = str(uuid.uuid4())
        policy_details['id'] = None
        policy_details['description'] = f"{policy_details['name']} dummy description."

        create_policy_response = policy_api.create_policy(org_key=SESSION.get_user_org_key(), body=policy_details)
        assert create_policy_response.ok, EXPECTED_CALL_TO_SUCCEED
        policy_id = json.loads(create_policy_response.content).get('id')
        CREATED_POLICY = policy_id

        device = sim_endpoint(SESSION, suffix=str(uuid.uuid4()), device_type=SensorDeviceType.MAC)
        CREATED_ENDPOINT = device.get("device_id")

        asset_group = create_asset_group(session=SESSION, policy_id=policy_id, name=f"dummy-{str(uuid.uuid4())}")
        CREATED_ASSET_GROUP = asset_group.get("id")
        time.sleep(20)

        # manually assign policy override
        add_policy_override(SESSION, policies[0].get("id"), device.get('device_id'))
        time.sleep(20)

        # manually add endpoint to asset group
        add_asset_group_to_endpoint(SESSION, asset_group.get("id"), device.get('device_id'))
        time.sleep(20)

        reevaluate([ CREATED_ENDPOINT ], CREATED_POLICY)
    except Exception as e:
        print(f"Exception: {e}")
        assert False
    finally:
        if CREATED_ENDPOINT is not None:
            delete_sensor(SESSION, CREATED_ENDPOINT)

        if CREATED_ASSET_GROUP is not None:
            delete_asset_group(SESSION, CREATED_ASSET_GROUP)

        if CREATED_POLICY is not None:
            delete_policy(SESSION, CREATED_POLICY)


@retry(retry=retry_if_exception_type(), reraise=True, stop=stop_after_attempt(20), wait=wait_fixed(5))
def reevaluate(asset_ids, policy_id):
    response = SESSION.post(
        url=f"{LOCAL_PAS_HOST}/private/orgs/{SESSION.get_user_org_key()}/reevaluate",
        data=json.dumps({
            "asset_ids": asset_ids
        })
    )
    assert response.ok, EXPECTED_CALL_TO_SUCCEED

    if policy_id:
        process, output, error = execute(["cat", "/tmp/app.log"])
        assert f"Appservices policy assignment API was called to assign policy {policy_id} for devices [{asset_id}] in org {SESSION.get_user_org_key()}" in output

    return response
